/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <fstream>
#include <Texture.hpp>
#include <algorithm>

namespace cpe
{


void Texture::export_ppm(const std::string& filename,const unsigned int& size_1,const unsigned int& size_2,const std::vector <float>& red_channel,const std::vector<float>& green_channel,const std::vector<float>& blue_channel)
{
    std::ofstream stream(filename.c_str(),std::ofstream::out);
    if(!stream.good())
        throw std::string("Error in Texture::export_ppm("+filename+"...) : cannot open file \n");

    //magic number
    stream<<"P3 \n";
    //size
    stream<<size_2<<" "<<size_1<<" \n";
    //color number
    stream<<"255"<<std::endl;

    //check if max is <1
    float multiplier=1.0;
    if( *std::max_element(red_channel.begin(),red_channel.end()) <= 1.0 )
        multiplier=255.0;

    for(unsigned int k2=0;k2<size_2;++k2)
    {
        for(unsigned int k1=0;k1<size_1;++k1)
        {
            int r=std::min(std::max(static_cast<int>(multiplier*red_channel  [k1+size_1*k2]),0),255);
            int g=std::min(std::max(static_cast<int>(multiplier*green_channel[k1+size_1*k2]),0),255);
            int b=std::min(std::max(static_cast<int>(multiplier*blue_channel [k1+size_1*k2]),0),255);

            stream<<r<<" "<<g<<" "<<b<<std::endl;
        }
    }
    stream.close();

}

void Texture::export_ppm(const std::string& filename,const unsigned int& size_1,const unsigned int& size_2,const std::vector <cpe::color>& color_vector)
{
    std::ofstream stream(filename.c_str(),std::ofstream::out);
    if(!stream.good())
        throw std::string("Error in Texture::export_ppm("+filename+"...) : cannot open file \n");

    //magic number
    stream<<"P3 \n";
    //size
    stream<<size_2<<" "<<size_1<<" \n";
    //color number
    stream<<"255"<<std::endl;

    for(unsigned int k2=0;k2<size_2;++k2)
    {
        for(unsigned int k1=0;k1<size_1;++k1)
        {
            const cpe::color& c=color_vector[k1+size_1*k2];
            stream<<c.r()<<" "<<c.g()<<" "<<c.b()<<std::endl;
        }
    }
    stream.close();

}

}
