
#include <cmath>
#include <iostream>
#include <sstream>
#include <Volume.hpp>
#include <Texture.hpp>
#include <exception_cpe.hpp>

using namespace cpe;

std::string get_string(const int& k);


int main(int argc,char *argv[])
{

  std::cout<<"start "<<argv[0]<<" with "<<argc<<" arg"<<std::endl;

  try
  {

      //************************//
      //Load
      //************************//
      Volume vol=Volume::load_v4d("data/head_ct_small.4d");

      //valeurs entre 0 et 1
      vol.normalize_data_to_one();

      //lissage du volume
      for(int k=0;k<2;++k)
          vol=vol.smoothed();

      //echantillonnage
      vol=vol.resampled_square(60);




      //************************//
      //export slices
      //************************//
      std::vector<float> sx=vol.slice_x(vol.size_x()/2.0);
      Texture::export_ppm("output/slice_x.ppm",vol.size_y(),vol.size_z(),sx,sx,sx);

      std::vector<float> sy=vol.slice_y(vol.size_y()/2.0);
      Texture::export_ppm("output/slice_y.ppm",vol.size_x(),vol.size_z(),sy,sy,sy);

      std::vector<float> sz=vol.slice_z(vol.size_z()/2.0);
      Texture::export_ppm("output/slice_z.ppm",vol.size_x(),vol.size_y(),sz,sz,sz);


//      for(unsigned int k=0;k<vol.size_z();++k)
//      {
//          std::cout<<k<<","<<vol.size_z()<<std::endl;
//          std::vector<float> sz=vol.slice_z(k);
//          Texture::export_ppm("output/slice_z_"+get_string(k)+".ppm",vol.size_x(),vol.size_y(),sz,sz,sz);
//      }




      //************************//
      //Integration
      //************************//

      //number of rotation steps
      int N=5;
      for(int counter=0;counter<N;++counter)
      {
          std::string number=get_string(counter);

          std::cout<<"export file "<<counter+1<<" / "<<N<<std::endl;
          double angle=static_cast<double>(counter)/static_cast<double>(N-1)*M_PI*2+M_PI/2.0;
          Volume vol2=vol.rotated_z(angle);


          //MIP
          std::vector<float> val=vol2.mip();
          Texture::export_ppm("output/mip_"+number+".ppm",vol.size_y(),vol.size_z(),val,val,val);

          //Ray Casting
          std::vector<cpe::color> val_couleur=vol2.ray_cast();
          Texture::export_ppm("output/ray_"+number+".ppm",vol.size_y(),vol.size_z(),val_couleur);
      }


  }
  catch(const cpe::exception_cpe& e)
  {
      std::cout<<"================================"<<std::endl;
      std::cout<<"Exception catch"<<std::endl;
      std::cout<<e.info()<<std::endl;
  }

  return 0;
}

std::string get_string(const int& k)
{

    std::stringstream stream_number;stream_number<<k;
    std::string number(stream_number.str());

    if(k<10)
        number="00"+number;
    else if(k<100)
        number="0"+number;
    return number;

}
